@@ -24,13 +24,14 @@ pub mod config;
 pub mod embed;
 /// Error handling.
 pub mod error;
-/// GitHub client.
-#[cfg(feature = "github")]
-pub mod github;
 /// Common release type.
 pub mod release;
-#[cfg(feature = "repo")]
+/// Remote handler.
+#[cfg(any(feature = "github", feature = "gitlab", feature = "bitbucket"))]
+#[allow(async_fn_in_trait)]
+pub mod remote;
 /// Git repository.
+#[cfg(feature = "repo")]
 pub mod repo;
 /// Template engine.
 pub mod template;
