@@ -189,7 +189,15 @@ fn generate_changelog() -> Result<()> {
 			timestamp: 0,
 			previous:  None,
 			#[cfg(feature = "github")]
-			github: git_cliff_core::github::GitHubReleaseMetadata {
+			github: git_cliff_core::remote::RemoteReleaseMetadata {
+				contributors: vec![],
+			},
+			#[cfg(feature = "gitlab")]
+			gitlab: git_cliff_core::remote::RemoteReleaseMetadata {
+				contributors: vec![],
+			},
+			#[cfg(feature = "bitbucket")]
+			bitbucket: git_cliff_core::remote::RemoteReleaseMetadata {
 				contributors: vec![],
 			},
 		},
@@ -217,7 +225,15 @@ fn generate_changelog() -> Result<()> {
 			timestamp: 0,
 			previous:  None,
 			#[cfg(feature = "github")]
-			github: git_cliff_core::github::GitHubReleaseMetadata {
+			github: git_cliff_core::remote::RemoteReleaseMetadata {
+				contributors: vec![],
+			},
+			#[cfg(feature = "gitlab")]
+			gitlab: git_cliff_core::remote::RemoteReleaseMetadata {
+				contributors: vec![],
+			},
+			#[cfg(feature = "bitbucket")]
+			bitbucket: git_cliff_core::remote::RemoteReleaseMetadata {
 				contributors: vec![],
 			},
 		},
