@@ -1,10 +1,12 @@
+use clap::Parser;
 use git_cliff::args::Opt;
+use git_cliff::logger;
+use git_cliff_core::error::Result;
 use std::env;
 use std::process;
-use structopt::StructOpt;
 
-fn main() {
-	let args = Opt::from_args();
+fn main() -> Result<()> {
+	let args = Opt::parse();
 	if args.verbose == 1 {
 		env::set_var("RUST_LOG", "debug");
 	} else if args.verbose > 1 {
@@ -12,7 +14,7 @@ fn main() {
 	} else if env::var_os("RUST_LOG").is_none() {
 		env::set_var("RUST_LOG", "info");
 	}
-	pretty_env_logger::init();
+	logger::init()?;
 	match git_cliff::run(args) {
 		Ok(_) => process::exit(0),
 		Err(e) => {
